define(function() {
	var res_path = "./app/res/";

	var baseInfo = null;
	var clientVersion = null; //客户端版本号
	var allPackageNames = null;//保存所有应用程序的包名
	var installedAppIndex = [];//已安装的应用程序下标
    var uninstalledAppIndex = [];//未安装的应用程序下标
    var appDownloadInfo = null;//应用程序的下载进度信息
    var canSwitchAppsIndex = null;//当前可以在卡片进行切换展示的20个应用下标
    var currentShowAppsIndex = [];//当前正在显示的应用下标
    var currentPostion = 0;//用于记录当前显示到数组第几个应用
    var PAGE_SHOW_NUM = 4;//一屏显示的应用数目
    var MAX_PAGE_NUM = 5;//换一换最多屏数

	var appCardData = null; //天气卡片数据
	var appCardDiv = null;
	var isAppCardCreate = false; //标识应用卡片视图是否已经创建
	var isAppCardRequested = false; //标识应用卡片是否请求过
	var isAppCardRequesting = false; //标识应用卡片是否正在请求
    var requestCondition = 0;//0:无网络，不需要请求;1:不判断网络，直接请求;2：判断网络，再请求
    var VERSION_THRESHOLD = 1909; //老版本的阈值
    /** 应用列表项元素的缓存*/
    var appItemCache = {};

    var zdOffDom = null;
    var speechCaseDom = null;
    var appCard = null;

    /**安装按钮的不同状态的文本*/
    var installButtonText = {
        "install":"安装",
        "open":"打开"
    };

    var cpaCodeKey = "appsearch";
    var opCodeKey = { //记录操作日志的key
	    "item":"item",
	    "category":"category",
        "switchBtn":"switchBtn",
        "moreBtn": "moreBtn"
	};
	
	function checkString(s) {
		if (s && s.length > 0) {
		    return true;
		}
		return false;
	}
	
	function filterRequestResult(requestResult) {
        console.log("filterRequestResult(), requestResult is " + requestResult);
	    if (!requestResult || !requestResult.content) {
	        return null;
	    }
	    var content = requestResult.content;
	    if (!content.apps || content.apps.length < PAGE_SHOW_NUM || !content.category || content.category.length < 3 || !content.bottomButton || content.bottomButton.length < 1) {
	        return null;
	    }
	    
	    try {
	        var filterResult = {};
            filterResult.apps = [];
            for (var i = 0; i < content.apps.length; i ++) {
                var app = content.apps[i];
                if (app && checkString(app.id) && checkString(app.packageName) && checkString(app.autoOpen) 
                    && checkString(app.name) && checkString(app.imgUrl) && checkString(app.desc) && checkString(app.downUrl)) {
                    filterResult.apps.push(app);
                }
            }
            if (filterResult.apps.length < PAGE_SHOW_NUM) {
                return null;
            }
            
            filterResult.category = [];
            for (var i = 0; i < content.category.length; i ++) {
                var category = content.category[i];
                if (category && checkString(category.name) && checkString(category.url)) {
                    filterResult.category.push(category);
                }
            }
            if (filterResult.category.length < 3) {
                return null;
            }
            
            filterResult.bottomButton = [];
            for (var i = 0; i < content.bottomButton.length; i ++) {
                var bottomButton = content.bottomButton[i];
                if (bottomButton && checkString(bottomButton.name) && checkString(bottomButton.url)) {
                    filterResult.bottomButton.push(bottomButton);
                }
            }
            if (filterResult.bottomButton.length < 1) {
                return null;
            }
            
            return filterResult;
	    } catch (err) {
	        return null;
	    }
	}

	function handleRequestSuccess(requestResult) {
		console.log("handleRequestSuccess, requestResult is " + requestResult);

        isAppCardRequesting = false;

		/**
		 * 根据返回不同的状态进行不同处理
		 */
		try {
		    var requestResultJson = JSON.parse(requestResult);
		    var status = requestResultJson.status;
            var errorCode = requestResultJson.errorCode;
            console.log("status is " + status + ", errorCode is " + errorCode);
            if ("000000" == errorCode) { //数据成功返回
                //解析返回结果
                var filterResult = filterRequestResult(requestResultJson);
                console.log("filterResult is " + JSON.stringify(filterResult));
                if (filterResult) {
                    //回调给框架请求成功返回
                    listener.callBack(appCardData.id, CARD_CONTENT_REQUEST_SUCCESS);

                    //更新缓存数据
                    appCardData.content = filterResult;
                    cacheMgr.setCache(appCardData.id, JSON.stringify(appCardData));

                    //扫描返回的应用数组，获取应用的是否安装以及下载进度的信息，同时生成能够当前能够切换的20个应用
                    scanApps();

                    //刷新视图
                    appCardDiv.refreshCardView();

                    isAppCardRequested = true;
                    return;
                } 
            } else { //其它错误统一处理
                console.log("error code is not 000000, error code is " + errorCode);
            }  
		} catch(e) {
		    console.log("JSON parse exception");
		}

        //通知框架请求返回失败
        listener.callBack(appCardData.id, CARD_CONTENT_REQUEST_FAILE);
	}
	
	function handleRequestFail() {
	    console.log("handleRequestFail()");
	    //通知框架请求返回失败
	    listener.callBack(appCardData.id, CARD_CONTENT_REQUEST_FAILE);

        isAppCardRequesting = false;
	}

	function startRequest(isNeedjudgeNet) {
	    console.log("startRequest(), isNeedjudgeNet is " + isNeedjudgeNet);
        if (isAppCardRequesting) {
            console.log("app card is requesting");
            return;
        }
        isAppCardRequesting = true;

        if (isNeedjudgeNet) {
            //获取网络状态
            var netWorkResultString = exec("SystemComponents", "isNetWorkAvailable", []);
            var netWorkResult = getResultMessage(netWorkResultString);
            console.log("netWorkResult is " + netWorkResult);
            //若网络未连接，则回调给框架请求失败
            if (netWorkResult == false) {
                //回调请求错误给框架
                listener.callBack(appCardData.id, CARD_CONTENT_REQUEST_FAILE);
                return;
            }
        }

        var requestOptions = {};
        if (baseInfo && baseInfo.businessRequestUrl) {
            var timeStr = new Date().valueOf();
            requestOptions.url = baseInfo.businessRequestUrl + "?c=1007&gz=0&v=1.0&t=" + timeStr;
        } else {
            listener.callBack(appCardData.id, CARD_CONTENT_REQUEST_FAILE);
            return;
        }

        requestOptions.onSuccess = handleRequestSuccess;
        requestOptions.onFail = handleRequestFail;

		//requestOptions.url = "http://192.168.77.214/Client_Test_developer/Default.aspx" + "?c=app&gz=0&v=1.0&t=2014";

		var base = organizeJsonBaseData(baseInfo);
        if(!base){
            base = {};
        }
		var param = {
			"id" : "app",
			"version" : "" + appCardData.version,
            "extendParam":{}
		};
		var requestJson = {
			"cmd" : "1007",
			"base" : base,
			"param" : param
		};
		console.log("requestJson is " + JSON.stringify(requestJson));

		requestOptions.data = JSON.stringify(requestJson);

		sendJsonAjaxRequest(requestOptions);
	}
    
    /**
     *创建当个推荐应用区域 
     */
    function createRecommendAppDiv(recommendAppData, position) {
        var id = recommendAppData.id;
        //如果已经创建好，则直接返回
        if (id && appItemCache[id]) {
            console.log("app div already exists!");
            //防止当某个应用安装后，其他应用位置发生改变，此时操作日志的记录就出现问题
            appItemCache[id].opCodeKey = opCodeKey.item + position;
            return appItemCache[id];
        }

        console.log("createRecommendAppDiv(), recommendAppData is " + JSON.stringify(recommendAppData));
        var downUrl = recommendAppData.downUrl;
        var imageUrl = recommendAppData.imgUrl;
        var name = recommendAppData.name;
        var packageName = recommendAppData.packageName;
        var isAutoOpen = (recommendAppData.autoOpen == 0 ? false : true);
        var entry = "card_app";
        var isInstalled = recommendAppData.isInstalled;

        //最外层的div
        var recommendAppDiv = document.createElement("div");
        recommendAppDiv.className = "css_recommend_app_div";
        recommendAppDiv.downUrl = downUrl;
        recommendAppDiv.name = name;
        recommendAppDiv.packageName = packageName;
        recommendAppDiv.isAutoOpen = isAutoOpen;
        recommendAppDiv.entry = entry;
        recommendAppDiv.opCodeKey = opCodeKey.item + position;
        if (id && id.length > 0) { //保存到缓存中
            appItemCache[id] = recommendAppDiv;
        }

        
        //创建左边图片区域
        var appImgOutDiv = document.createElement("div");
        appImgOutDiv.className = "css_app_img_out_div";

        var appImg = document.createElement("img");
        appImg.className = "css_app_img";
        appImg.src = imageUrl;
        appImg.onerror = function() {
            console.log("app image load error!");
            this.style.display = "none";
            if (id && id.length > 0) {
                // 图片加载失败时，清除相应的缓存
                console.log("clear icon cache, id is " + id);
                appItemCache[id] = null;
            }
        };

        appImgOutDiv.appendChild(appImg);
        recommendAppDiv.appendChild(appImgOutDiv);
        
        //创建右边的应用名称和安装按钮
        var rightDiv = document.createElement("div");
        rightDiv.className = "css_app_right_div";
        
        var appName = document.createElement("div");
        appName.innerHTML = name;
        appName.className = "css_app_name_div";
        
        var appInstallButtonDiv = document.createElement("div");
        appInstallButtonDiv.className = "css_app_button_div";
        
        var appInstallButton = document.createElement("div");
        if (clientVersion <= VERSION_THRESHOLD) {
            appInstallButton.innerHTML = installButtonText.install;
        } else if (isInstalled) {
            appInstallButton.innerHTML = installButtonText.open;
        } else if (appDownloadInfo[downUrl]) {
            var status = appDownloadInfo[downUrl].status;
            if (!status || status == "status_finished") {
                appInstallButton.innerHTML = installButtonText.install;
            } else {
                if (appDownloadInfo[downUrl].progress) {
                    appInstallButton.innerHTML = Math.floor(appDownloadInfo[downUrl].progress * 100) + "%";
                } else {
                    appInstallButton.innerHTML = installButtonText.install;
                }
            }
        } else {
            appInstallButton.innerHTML = installButtonText.install;
        }
        appInstallButton.className = "css_app_button";
        appInstallButtonDiv.backE = appInstallButton;
        appInstallButtonDiv.appendChild(appInstallButton);
        
        rightDiv.appendChild(appName);
        rightDiv.appendChild(appInstallButtonDiv);
        recommendAppDiv.appendChild(rightDiv);
        
        //创建底部的应用简介
        var appDes = document.createElement("div");
        appDes.className = "css_app_des_div";
        appDes.innerHTML = recommendAppData.desc;
        recommendAppDiv.appendChild(appDes);

        //添加安装按钮的点击事件
        var installButtonDivMove = false;
        appInstallButtonDiv.addEventListener("touchstart", function() {
            event.stopPropagation();
            console.log("appInstallButtonDiv touchstart!");
            moveOverCheck.start(event);
            installButtonDivMove = false;
            appInstallButton.style.backgroundColor = "#4282f7";
            appInstallButton.style.color = "#ffffff";
        }, false);
        appInstallButtonDiv.addEventListener("touchmove", function() {
            event.stopPropagation();
            console.log("appInstallButtonDiv touchmove!");
            if (moveOverCheck.check(event)) {
                installButtonDivMove = true;
            }
            appInstallButton.style.backgroundColor = "";
            appInstallButton.style.color = "";
        }, false);
        appInstallButtonDiv.addEventListener("touchend", function() {
            event.stopPropagation();
            console.log("appInstallButtonDiv touchend!");
            if (false == installButtonDivMove) {
                if (clientVersion > VERSION_THRESHOLD) {
                    //调用下载能力接口进行处理
                    exec("DownloadComponents", "handleAppInstallAction", [recommendAppDiv.downUrl, recommendAppDiv.name, recommendAppDiv.packageName, recommendAppDiv.isAutoOpen, recommendAppDiv.entry]);
                } else {
                    //调用浏览器打开downloadUrl
                    exec("UIComponents", 'openBrowser', [ "", recommendAppDiv.downUrl]);
                }
                //记录稽核日志及操作日志
                exec("WidgetMMPContainerComponents", "appenCardCpaLog", [cpaCodeKey]);
                exec("WidgetMMPContainerComponents", "appenCardOpLog", [cardOpCode.app, recommendAppDiv.opCodeKey]);
            }
            appInstallButton.style.backgroundColor = "";
            appInstallButton.style.color = "";
        }, false);

        recommendAppDiv.refreshButton = function(content) {
            console.log("recommendAppDiv.refreshButton(), content is " + content);
            appInstallButton.innerHTML = content;
        };
        
        return recommendAppDiv;
    }
    
    /**
     *创建底部的按钮区域 
     */
    function createAppCardBottom(bottomButtonData) {
        console.log("createAppCardBottom(), bottomButtonData is " + bottomButtonData);
        
        if (!bottomButtonData || bottomButtonData.length <= 0) {
            console.log("bottomButtonData is illegal!");
        }
        
        var appCardBottom = document.createElement("div");
        
        //换一换按钮
        var switchButton = document.createElement("div");
        switchButton.className = "css_app_switch_button";
        switchButton.innerHTML = "换一换";
        appCardBottom.appendChild(switchButton);
        
        //更多按钮
        var getMoreButton = document.createElement("div");
        getMoreButton.className = "css_app_get_more_button";
        getMoreButton.innerHTML = bottomButtonData[0].name;
        getMoreButton.detailUrl = bottomButtonData[0].url;
        appCardBottom.appendChild(getMoreButton);
        
        //添加点击事件
        var cardBottomMove = false;
        var target = null;
        appCardBottom.addEventListener("touchstart", function() {
            event.stopPropagation();
            console.log("appCardBottom touchstart!");
            moveOverCheck.start(event);
            cardBottomMove = false;
            target = event.target;
            target.style.backgroundColor = "#efefef";
        }, false);
        appCardBottom.addEventListener("touchmove", function() {
            event.stopPropagation();
            console.log("appCardBottom touchmove!");
            if (moveOverCheck.check(event)) {
                cardBottomMove = true;
            }
            target.style.backgroundColor = "";
        }, false);
        appCardBottom.addEventListener("touchend", function() {
            event.stopPropagation();
            console.log("appCardBottom touchend!");
            if (false == cardBottomMove) {
                if (target) {
                    if (target == getMoreButton) {//更多应用,直接打开url
                        if (target.detailUrl) {
                            exec("UIComponents", 'openBrowser', [ "", target.detailUrl]);
                        }

                        //记录稽核日志及操作日志
                        exec("WidgetMMPContainerComponents", "appenCardCpaLog", [cpaCodeKey]);
                        exec("WidgetMMPContainerComponents", "appenCardOpLog", [cardOpCode.app, opCodeKey.moreBtn]);
                    } else if (target == switchButton) {//换一换
                        switchApps();

                        //记录稽核日志及操作日志
                        exec("WidgetMMPContainerComponents", "appenCardCpaLog", [cpaCodeKey]);
                        exec("WidgetMMPContainerComponents", "appenCardOpLog", [cardOpCode.app, opCodeKey.switchBtn]);
                    }
                }
            }
            target.style.backgroundColor = "";
            target = null;
        }, false);

        appCardBottom.refreshBottomView = function() {
            var bottomButton = appCardData.content.bottomButton;
            if (!bottomButton || bottomButton.length <= 0) {
                 return;
            }
            console.log("appCardBottom.refreshBottomView bottomButton is " + JSON.stringify(bottomButton));
            getMoreButton.innerHTML = bottomButton[0].name;
            getMoreButton.detailUrl = bottomButton[0].url;
        };
        
        return appCardBottom;
    }

    function switchApps() {
        console.log("switchApps(), currentPostion is " + currentPostion);
        //若最大可显示的屏数是1，则“换一换”不做任何处理
        if (MAX_PAGE_NUM <= 1) {
            return;
        }

        //计算点击“换一换”后，下次要展示的数据在可切换展示的应用数组中的起始和终止下标
        var startPos =  (currentPostion + 1)%(MAX_PAGE_NUM*PAGE_SHOW_NUM);
        var endPos = startPos + PAGE_SHOW_NUM - 1;
        currentPostion = endPos;
        console.log("startPos is " + startPos + ", endPos is " + endPos + ", currentPostion is " + currentPostion);

        currentShowAppsIndex = canSwitchAppsIndex.slice(startPos, endPos+1);
        console.log("canSwitchAppsIndex is " + canSwitchAppsIndex);
        appCardDiv.refreshRecommendAppsView(currentShowAppsIndex);
    }
    
    /**
     *创建应用卡片的分类 
     */
    function createAppCardCategory(appCategory) {
       console.log("createAppCardCategory(), appCategory is " + JSON.stringify(appCategory));
       
       if (!appCategory || appCategory.length <= 0) {
           console.log("appCategory is illegal!");
           return;
       }
       
       var appCategoryDiv = document.createElement("div");
       appCategoryDiv.className = "css_app_category_out_div";
       
       for (var i = 0; i < appCategory.length; i++) {
           var category = appCategory[i];
           var categoryDiv = document.createElement("div");
           categoryDiv.className = "css_app_category_div";
           categoryDiv.innerHTML = category.name;
           categoryDiv.detailUrl = category.url;
           categoryDiv.opCodeKey = opCodeKey.category + (i+1);
           if (i == 1) { //中间的分类设置左右间距
               categoryDiv.className += " css_app_category_margin";
           }
           appCategoryDiv.appendChild(categoryDiv);
       }
       
       //添加点击事件
       var categoryDivMove = false;
       var target = null;
       appCategoryDiv.addEventListener("touchstart", function() {
            event.stopPropagation();
            console.log("appCategoryDiv touchstart!");
            moveOverCheck.start(event);
            categoryDivMove = false;
            target = event.target;
            if (target && target.parentNode == appCategoryDiv) {
                target.style.backgroundColor = "#e0e0e0";
            }
        }, false);
        appCategoryDiv.addEventListener("touchmove", function() {
            event.stopPropagation();
            console.log("appCategoryDiv touchmove!");
            if (moveOverCheck.check(event)) {
                categoryDivMove = true;
            }
            if (target && target.parentNode == appCategoryDiv) {
                target.style.backgroundColor = "";
            }
        }, false);
        appCategoryDiv.addEventListener("touchend", function() {
            event.stopPropagation();
            console.log("appCategoryDiv touchend!");
            if (false == categoryDivMove) {
                if (target && target.parentNode == appCategoryDiv) {
                    if (target.detailUrl) {
                        exec("UIComponents", 'openBrowser', [ "", target.detailUrl]);
                    }

                    //记录稽核日志及操作日志
                    exec("WidgetMMPContainerComponents", "appenCardCpaLog", [cpaCodeKey]);
                    exec("WidgetMMPContainerComponents", "appenCardOpLog", [cardOpCode.app, target.opCodeKey]);

                    target.style.backgroundColor = "";
                }
            }
            target = null;
        }, false);

        /**
         * 对外提供的刷新界面的接口
         */
        appCategoryDiv.refreshCategoryView = function() {
            var appCategory = appCardData.content.category;
            console.log("appCategoryDiv.refreshCategoryView(), appCategory is " + JSON.stringify(appCategory));
            for (var i = 0; i < appCategory.length; i ++) {
                var childDiv = appCategoryDiv.childNodes[i];
                childDiv.innerHTML = appCategory[i].name;
                childDiv.detailUrl = appCategory[i].url;
            }
        };
       
       return appCategoryDiv;
    }
    
	/**
	 *创建推荐应用程序的区域 
	 */
	function createRecommendAppsDiv(recomendAppsIndex) {
	   if (!recomendAppsIndex || recomendAppsIndex.length <= 0) {
	       console.log("recomendAppsIndex is illegal!");
	       return;
	   }
       console.log("createRecommendAppsDiv(), recomendAppsIndex is " + JSON.stringify(recomendAppsIndex));
	   
	   var recommendAppsDiv = document.createElement("div");
	   //循环构建
	   for(var i = 0; i < recomendAppsIndex.length; i ++){
	       var recommendAppData = appCardData.content.apps[recomendAppsIndex[i]];
	       var recommendAppDiv = createRecommendAppDiv(recommendAppData, i+1);
	       recommendAppsDiv.appendChild(recommendAppDiv);
	   }

        /**
         * 对外提供的刷新界面的接口
         */
       recommendAppsDiv.refreshRecommendAppsView = function(refreshAppsDataIndex) {
            console.log("recommendAppsDiv.refreshRecommendAppsView()");
            recommendAppsDiv.innerHTML = "";
            for (var i = 0; i < refreshAppsDataIndex.length; i ++) {
                var appData = appCardData.content.apps[refreshAppsDataIndex[i]];
                var appDiv = createRecommendAppDiv(appData, i+1);
                recommendAppsDiv.appendChild(appDiv);
            }
       };

        /**
         * 对外提供的刷新按钮状态和下载进度的接口
         */
        recommendAppsDiv.refreshButton = function(mark, content) {
            console.log(" recommendAppsDiv.refreshButton(), mark is " + mark + ", content is " + content);
            for (var i = 0; i < canSwitchAppsIndex.length; i ++) {
                var appData = appCardData.content.apps[canSwitchAppsIndex[i]];
                if (appData.downUrl == mark || appData.packageName == mark) {
                    if (appData.id && appData.id.length > 0) {
                        var appDiv = appItemCache[appData.id];
                        console.log("appDiv is " + appDiv);
                        if (appDiv) {
                            appDiv.refreshButton(content);
                        }
                    }
                }
            }
        };

	   
	   return recommendAppsDiv;
	}
	
	/**
	 *创建天气卡片的title区域
	 */
	function createAppCardTitle(appData) {
		console.log("createAppCardTitle, appData is " + JSON.stringify(appData));

		var appCardTitle = document.createElement("div");
		appCardTitle.className = "css_card_title_bar";

		//左边的标题区域
		var leftTitle = document.createElement("div");
		leftTitle.className = "css_card_title_text";
        leftTitle.innerHTML = appCardData.name;
        
		appCardTitle.appendChild(leftTitle);
		return appCardTitle;
	}
	
	function createAppCardView() {
		console.log("createAppCardView()");
		
		var appData = null;
		if (appCardData) {
			appData = appCardData.content;
		}
		console.log("appData is " + JSON.stringify(appData));
        
        var appArea = document.createElement("div");
        appArea.className = "css_card";
        
		//创建应用卡片的title区域
		//var appCardTitle = createAppCardTitle(appData);
        var appCardTitle = createCardMainTitle(appCardData.id, appCardData.name,appCardData.caseList[0],titleResponseListener);
		if (appCardTitle) {
            appArea.appendChild(appCardTitle);
        }


		//创建应用卡片的推荐应用区域
		var appCardRecommendApps = createRecommendAppsDiv(currentShowAppsIndex);
		appArea.appendChild(appCardRecommendApps);
		
		//创建应用卡片的分类
		var appCardCategory = createAppCardCategory(appData.category);
		appArea.appendChild(appCardCategory);
		
		//创建分割线
		var appCardSeparator = document.createElement("div");
		appCardSeparator.className = "css_app_card_separator";
		appArea.appendChild(appCardSeparator);
		
		//创建底部换一换和更多应用区域
		var appCardBottom = createAppCardBottom(appData.bottomButton);
		appArea.appendChild(appCardBottom);

        //创建底部的阴影区
        var appCardBottomShadow = document.createElement("div");
        appCardBottomShadow.className = "css_card_bottom_shadow";
        appArea.appendChild(appCardBottomShadow);
		
		appCardDiv.refreshCardView = function() {
            console.log("appCardDiv.refreshCardView");
		    //刷新推荐应用的视图
		    appCardRecommendApps.refreshRecommendAppsView(currentShowAppsIndex);
		    //刷新应用分类的视图
		    appCardCategory.refreshCategoryView();
		    //刷新应用底部的视图
		    appCardBottom.refreshBottomView();
		};

        appCardDiv.refreshRecommendAppsView = function(refreshAppsDataIndex) {
            console.log("appCardDiv.refreshRecommendAppsView(), refreshAppsDataIndex is " + refreshAppsDataIndex);
            //调用接口获取要换一换的应用是否安装以及下载信息
            if (clientVersion > VERSION_THRESHOLD) {
                getInstalledAndDownloadInfo(refreshAppsDataIndex);
            }

            //刷新界面
            appCardRecommendApps.refreshRecommendAppsView(refreshAppsDataIndex);
        };

        appCardDiv.refreshButton = function(mark, content) {
            appCardRecommendApps.refreshButton(mark, content);
        };

        //保存变量
        appCard = appArea;

        if (appCardDiv) {
            appCardDiv.appendChild(appArea);
            appCardDiv.showContentPage = showContentPage;
        }
		
		//设置已经创建的标识位为true
		isAppCardCreate = true;
	}
	
	function onResourceLoadFinish() {
		console.log("onResourceLoadFinish(), appCardData is " + JSON.stringify(appCardData));
		
		//获取基本信息以及客户端的版本号
		baseInfo = getBseInfo();
		if (baseInfo && baseInfo.base) {
		    var clientVersionString = String(baseInfo.base.clientver);
		    if (clientVersionString) {
		        var clientVersionArray = clientVersionString.split(".");
                if (clientVersionArray && clientVersionArray.length >= 3) {
                    clientVersion = parseInt(clientVersionArray[2]);
                    console.log("clientVersion is " + clientVersion);
                }
		    }
		}
		

        //扫描应用数组，获取应用的是否安装以及下载进度的信息，同时生成能够当前能够切换的20个应用
        scanApps();
		
		//创建应用卡片视图
		createAppCardView();

        //注册下载监听器
        exec("DownloadComponents", "registerDownloadListener",[1]);

		//请求应用数据
        if (1 == requestCondition) {
            startRequest(false);
        } else {
            if (2 == requestCondition) {
                startRequest(true);
            }  else {
                //没有网络，不需要请求，但是需要回调
                listener.callBack(appCardData.id, CARD_CONTENT_REQUEST_FAILE);
            }
        }
	}

    function getInstalledAndDownloadInfo(appsIndex) {
        console.log("getInstalledAndDownloadInfo(), appsIndex is " + appsIndex);
        var packageNames = new Array();
        var installedAppsIndex = new Array();
        var uninstalledAppsIndex = new Array();
        for (var i = 0; i < appsIndex.length; i ++) {
            var appData = appCardData.content.apps[appsIndex[i]];
            packageNames.push(appData.packageName);
        }

        //调用接口获取应用是否安装信息
        var appInstalledResult = getResultMessage(exec("AppComponents", "isAppInstalled", [packageNames]));
        console.log("appInstalledResult is " + JSON.stringify(appInstalledResult));
        for (var i = 0; i < packageNames.length; i ++) {
            var index = appsIndex[i];
            if (true == appInstalledResult[packageNames[i]]) {//安装
                installedAppsIndex.push(index);
                appCardData.content.apps[index].isInstalled = true;
            } else { //未安装
                uninstalledAppsIndex.push(index);
                appCardData.content.apps[index].isInstalled = false;
            }
        }

        //获取未安装应用程序的下载进度信息
        getAppsDownloadInfo(uninstalledAppsIndex);
    }

    /**
     * 扫描应用数组，获取应用的是否安装以及下载进度的信息，同时生成能够当前能够切换的20个应用
     * @returns {Array}
     */
    function scanApps() {
        //初始化操作,清空数组
        installedAppIndex.splice(0, installedAppIndex.length);
        uninstalledAppIndex.splice(0, uninstalledAppIndex.length);

        //遍历apps数组获取所有应用的包名
        allPackageNames = getAllAppPackageNames();
        console.log("allPackageNames is " + allPackageNames);

        if (clientVersion > VERSION_THRESHOLD) {
            //调用接口检查应用程序是否已安装
            var appInstalledResult = getResultMessage(exec("AppComponents", "isAppInstalled", [allPackageNames]));
            console.log("isAppInstalled result is " + JSON.stringify(appInstalledResult));
            for (var index = 0; index < allPackageNames.length; index ++) {
                if (true == appInstalledResult[allPackageNames[index]]) {
                    installedAppIndex.push(index);
                    appCardData.content.apps[index].isInstalled = true;
                } else {
                    uninstalledAppIndex.push(index);
                    appCardData.content.apps[index].isInstalled = false;
                }
            }
            console.log("installedAppIndex is " + installedAppIndex);
            console.log("uninstalledAppIndex is " + uninstalledAppIndex);

            //根据应用程序是否已安装的情况，去获取未安装应用程序的下载进度
            getAppsDownloadInfo(uninstalledAppIndex);

            //获取20个用来切换展示的应用
            canSwitchAppsIndex = getSwitchAppsIndex();
        } else {
            //旧版本，用来切换的20个app就是所有app中的前20个
            canSwitchAppsIndex = getSwitchAppsIndex();
        }

        currentShowAppsIndex = canSwitchAppsIndex.slice(0, PAGE_SHOW_NUM);
        currentPostion = PAGE_SHOW_NUM - 1;
        console.log("canSwitchAppsIndex is " + canSwitchAppsIndex + ", currentShowAppsIndex is " + currentShowAppsIndex + ", currentPostion is " + currentPostion);
    }

    /**
     * 获取用于切换展示的20个应用
     * @returns {Array}
     */
    function getSwitchAppsIndex() {
        var switchApps = [];
        //获取可展示的屏数
        var switchNum = Math.floor(allPackageNames.length/PAGE_SHOW_NUM);
        console.log("switchNum is " + switchNum);
        MAX_PAGE_NUM = 5;
        if (switchNum < MAX_PAGE_NUM) {
            MAX_PAGE_NUM = switchNum;
        }

        var totalNum = MAX_PAGE_NUM*PAGE_SHOW_NUM;
        if (clientVersion > VERSION_THRESHOLD) {
            //将未安装的应用添加到可切换的应用数组中
            for (var i = 0; i < uninstalledAppIndex.length; i ++) {
                if (switchApps.length >= totalNum) {
                    break;
                }
                switchApps.push(uninstalledAppIndex[i]);
            }

            //若可切换的应用数目没有达到可展示的屏数，则用已安装的应用进行补充
            var remainNum =  totalNum - switchApps.length;
            if (installedAppIndex.length > 0 && remainNum > 0) {
                for (var i = 0; i < remainNum; i ++) {
                    switchApps.push(installedAppIndex[i]);
                }
            }
        } else { //1909及之前的版本直接获取前totalNum个应用作为切换展示的应用
            for (var i = 0; i < totalNum; i ++) {
                switchApps.push(i);
            }
        }

        return switchApps;
    }

    /**
     * 获取所有推荐应用的包名
     * @returns {Array}
     */
	function getAllAppPackageNames() {
	    var packageNames = [];
        var apps = appCardData.content.apps;
	    for (var i = 0; i < apps.length; i ++) {
	        packageNames.push(apps[i].packageName);
	    }
	    return packageNames;
	}

    /**
     * 获取未安装的应用
     */
    function getUninstallApps(uninstalledAppsIndex) {
        var uninstallPackage = [];
        if (allPackageNames && allPackageNames.length > 0) {
            for (var i = 0; i < uninstalledAppsIndex.length; i ++) {
                uninstallPackage.push(allPackageNames[uninstalledAppsIndex[i]]);
            }
        }
        return uninstallPackage;
    }

    /**
     * 获取所有应用的下载信息
     */
	function getAppsDownloadInfo(appsIndex) {
        //获取apps的对应的下载地址
        var dowanloadUrlArray = [];
        for (var i = 0; i < appsIndex.length; i ++) {
             dowanloadUrlArray.push(appCardData.content.apps[appsIndex[i]].downUrl);
        }
        console.log("dowanloadUrlArray is " + dowanloadUrlArray);
        //调用本地接口获取未安装应用的下载进度
        if (dowanloadUrlArray && dowanloadUrlArray.length > 0) {
            appDownloadInfo = getResultMessage(exec("DownloadComponents", "queryTaskProgress", [dowanloadUrlArray]));
        }
        console.log("getAppsDownloadInfo(), appDownloadInfo is " + JSON.stringify(appDownloadInfo));
	}
	
	function createAppCard(initNetState) {
		var appCardId = appCardData.id;
		
		//读取缓存内容
		var cacheAppString = cacheMgr.getCache(appCardId);
		console.log("cacheAppString is " + cacheAppString);
		if (!cacheAppString) { //缓存为空，则使用默认数据
			cacheMgr.setCache(appCardId, JSON.stringify(appCardData));
		} else {
			if(typeof cacheAppString == "string") {
				try{
					var appCacheCardData = JSON.parse(cacheAppString);
					//检查缓存的有效性，即比较版本号
					//若版本号相同的话，则缓存有效，直接使用缓存
					//若版本号不相同的话，则缓存无效，这时需取缓存的“show”字段
					if (appCacheCardData.version == appCardData.version) {
                        appCardData.show = appCacheCardData.show;
                        if (appCacheCardData.content) {
                            appCardData.content = appCacheCardData.content;
                        }
					} else {
						appCardData.show = appCacheCardData.show; //保留卡片是否需要显示
						cacheMgr.setCache(appCardId, JSON.stringify(appCardData));
					}
				}catch(e){
				    cacheMgr.setCache(appCardId, JSON.stringify(appCardData));
					console.log("JSON parse exception");
				}
			}
		}
		
		//若卡片可以显示，则加载资源并创建视图
		console.log("appCardData is " + JSON.stringify(appCardData));
		if (appCardData && appCardData.show) {
            if (initNetState) {
                requestCondition = 1;
            } else {
                requestCondition = 0;
            }
			//加载css资源文件
			xcssParser.parseStyleFile("./app/business.x.css", onResourceLoadFinish);
		} else {
		    //卡片不需要显示时，也需要回调给框架请求成功
		    listener.callBack(appCardData.id, CARD_CONTENT_NO_REQUEST);
            requestCondition = 2;
		}
	}

	function createAppCardFrame(cardDefaultData, initNetState) {
		console.log("createAppCardFrame(), cardDefalutData is " + JSON.stringify(cardDefaultData));
		appCardData = cardDefaultData;
		
		appCardDiv = document.createElement("div");
		appCardDiv.id = appCardData.id;
		
		createAppCard(initNetState);

        appCardDiv.updateData = function(){
            console.log("appCardDiv.updateData(), appCardData.show is " + appCardData.show);
            if (appCardData.show) {
                startRequest(false);
            }
            else {
                if (!isAppCardRequesting) {
                    listener.callBack(appCardData.id, CARD_CONTENT_NO_REQUEST);
                }
            }
        };

        //更新卡片是否显示的状态
        appCardDiv.updateState = function(state) {
            console.log("appCardDiv.updateCardState, state is " + state);
            if (true == state || "true" == state) { //显示卡片
                if (isAppCardCreate) { //应用卡片视图已创建
                    console.log("isAppCardRequested is " + isAppCardRequested);
                    appCardDiv.style.display = "block";
                    if (!isAppCardRequested) {
                        //未成功请求过，不仅需要显示视图，而且还需进行请求
                        if (!isAppCardRequesting) {
                            startRequest();
                        }
                    }
                } else { //应用卡片视图未创建
                    xcssParser.parseStyleFile("./app/business.x.css", onResourceLoadFinish);
                }
            } else if (false == state || "false" == state) { //不显示卡片
                appCardDiv.style.display = "none";
            }

            //更新缓存
            appCardData.show = state;
            cacheMgr.setCache(appCardData.id, JSON.stringify(appCardData));
        };

        appCardDiv.date_type = "card";
        appCardDiv.initShowState = appCardData.show;

		return appCardDiv;
	}

    function operateCallBack(callBackId) {
        switch(callBackId) {
            case 0://取消
                if (zdOffDom) {
                    zdOffDom.style.display = "none";
                }
                appCard.style.display = "block";
                break;
            case 1://置顶
                if (zdOffDom) {
                    zdOffDom.style.display = "none";
                }
                appCard.style.display = "block";
                stickTopCardByDom(appCardDiv);
                break;
            case 2://关闭
                //显示关闭确认弹框
                showCloseConfirmDialogById(appCardData.id);
                break;

            default:
                break;
        }
    }
    function speechCaseCallBack(){
        if(zdOffDom) {
            zdOffDom.style.display = "none";
        }
        if(speechCaseDom) {
            speechCaseDom.style.display = "none";
        }
        appCard.style.display = "block";
    }
    function titleResponseListener(responseId){
        switch(responseId) {
            case 0://点击说法示例区域
                if(zdOffDom) {
                    zdOffDom.style.display = "none";
                }

                appCard.style.display = "none";
                if (speechCaseDom) {
                    speechCaseDom.style.display = "block";
                } else {
                    speechCaseDom = createSpeechCaseView(appCardData.id, appCardData.name, appCardData.caseList);
                    if (speechCaseDom) {
                        speechCaseDom.setCancelListener(speechCaseCallBack);
                        appCardDiv.appendChild(speechCaseDom);
                    }
                }
                break;
            case 1://点击置顶箭头按钮
                if(speechCaseDom) {
                    speechCaseDom.style.display = "none";
                }
                appCard.style.display = "none";

                if (zdOffDom) {
                    zdOffDom.style.display = "block";
                } else {
                    zdOffDom = createOperationView(appCardData.id, appCardData.name);
                    if (zdOffDom) {
                        zdOffDom.setOpListener(operateCallBack);
                        appCardDiv.appendChild(zdOffDom);
                    }
                }
                break;
            case 2://刷新，暂时没有实现
                break;
            default:
                break;
        }
    }
    function showContentPage(){
        if(zdOffDom) {
            zdOffDom.style.display = "none";
        }
        if(speechCaseDom) {
            speechCaseDom.style.display = "none";
        }
        appCard.style.display = "block";
    }

    /******下载进度及状态回调******************/
    function onDownloadStart(downloadUrl) {
        console.log("onDownloadStart(), downloadUrl is " + downloadUrl);
        appCardDiv.refreshButton(downloadUrl, "0%");
    }

    function onDownloadProgress(downloadUrl, receivedBytes, totalBytes) {
        console.log("onDownloadProgress(), downloadUrl is " + downloadUrl + "receivedBytes is " + receivedBytes + ", totalBytes is " + totalBytes);
        var progress = parseInt(receivedBytes*100/totalBytes) + "%";
        appCardDiv.refreshButton(downloadUrl, progress);
    }

    function onDownloadFail(downloadUrl, errorCode) {
        console.log("onDownloadFail(), downloadUrl is " + downloadUrl + ", errorCode is " + errorCode);
    }

    function onDownloadSuccess(downloadUrl) {
        console.log("onDownloadSuccess(), downloadUrl is " + downloadUrl);
        appCardDiv.refreshButton(downloadUrl, installButtonText.install);
    }

    function onDownloadCanceled(downloadUrl) {
        console.log("onDownloadCanceled(), downloadUrl is " + downloadUrl);
        appCardDiv.refreshButton(downloadUrl, installButtonText.install);
    }

    function onInstallSuccess(packageName) {
        console.log("onInstallSuccess(), packageName is " + packageName);
        appCardDiv.refreshButton(packageName, installButtonText.open);
    }

    window. onDownloadStart = onDownloadStart;
    window. onDownloadProgress = onDownloadProgress;
    window. onDownloadFail = onDownloadFail;
    window. onDownloadSuccess = onDownloadSuccess;
    window. onDownloadCanceled = onDownloadCanceled;
    window. onInstallSuccess = onInstallSuccess;

	return createAppCardFrame;
}); 